/*
 * Copyright (c) 2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** velodyneCntrlTaskCodels.c
 **
 ** Codels used by the control task velodyneCntrlTask
 **
 **/

#include <portLib.h>

/* standard librairies */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "server/velodyneHeader.h"
#include <velodyne/velodyneLib.h>

#include "velodyneBuffer.h"

extern velodyneReadTask task;

/*------------------------------------------------------------------------
 * velodyneStopAcquisitionCntrl  -  control codel of CONTROL request Stop
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
velodyneStopAcquisitionCntrl(int *report)
{
  SDI_F->running = 0;
  pthread_join(task.thr, NULL);
  pthread_mutex_destroy(&task.mtx);
  return OK;
}

/*------------------------------------------------------------------------
 * velodyneSetSaveParamsCntrl  -  control codel of request SetSaveParams
 *
 * Description: checks that the suffix number is positive
 * Report: OK
 *                 S_velodyne_BAD_SAVE_PARAMS
 *
 * Returns:    OK or ERROR
 */

STATUS
velodyneSetSaveParamsCntrl(velodyneSaveParams* saveParams, int *report)
{

  if (saveParams->num < 0) {
    fprintf (stderr, "velodyne: bad file suffix number\n");
    *report = S_velodyne_BAD_SAVE_PARAMS;
    return ERROR;
  }
  return OK;
}


/*------------------------------------------------------------------------
 * velodyneOneShotCntrl  -  control codel of request OneShot
 *
 * Description: checks that the provided angles are within correct bounds and
 * weel ordered
 * Report: OK
 *                 S_velodyne_BAD_ONE_SHOT_PARAMS
 *
 * Returns:    OK or ERROR
 */

STATUS
velodyneOneShotCntrl(velodyneOneShotParams* oneShotParams, int *report)
{

  if ((oneShotParams->firstAngle_degrees < -180.0)
      || (oneShotParams->firstAngle_degrees > 180)) {
    fprintf (stderr, "velodyne: respect -180 < firstAngle < 180 \n");
    *report = S_velodyne_BAD_ONE_SHOT_PARAMS;
    return ERROR;
  }
  if ((oneShotParams->lastAngle_degrees < -180.0)
      || (oneShotParams->lastAngle_degrees > 180)) {
    fprintf (stderr, "velodyne: respect -180 < lastAngle < 180 \n");
    *report = S_velodyne_BAD_ONE_SHOT_PARAMS;
    return ERROR;
  }
  return OK;
}
